﻿using EventStoreExample.Infrastructure;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Threading.Tasks;

namespace EventStoreExample.Controllers
{
	[Route("api/techitem/{id}")]
	[ApiController]
	public class TechItemController : ControllerBase
	{
		private readonly AggregateRepository _aggregateRepository;
		public TechItemController(AggregateRepository aggregateRepository)
		{
			_aggregateRepository = aggregateRepository;
		}

		[HttpPost, Route("create")]
		public async Task<IActionResult> Create(Guid id, [FromForm] string title)
		{
			var aggregate = await _aggregateRepository.LoadAsync<Core.TechItem>(id);
			aggregate.Create(id, title, "Pawel");

			await _aggregateRepository.SaveAsync(aggregate);

			return Ok();
		}

		[HttpPatch, Route("assign")]
		public async Task<IActionResult> Assign(Guid id, [FromForm] string assignedTo)
		{
			var aggregate = await _aggregateRepository.LoadAsync<Core.TechItem>(id);
			aggregate.Assign(assignedTo, "Krzysztof");

			await _aggregateRepository.SaveAsync(aggregate);

			return Ok();
		}


		[HttpPatch, Route("move")]
		public async Task<IActionResult> Move(Guid id, [FromForm] string activeColumn)
		{
			var aggregate = await _aggregateRepository.LoadAsync<Core.TechItem>(id);
			aggregate.Move(activeColumn, "Krzysztof");

			await _aggregateRepository.SaveAsync(aggregate);

			return Ok();
		}

		[HttpPatch, Route("complete")]
		public async Task<IActionResult> Complete(Guid id)
		{
			var aggregate = await _aggregateRepository.LoadAsync<Core.TechItem>(id);
			aggregate.Complete("Krzysztof");

			await _aggregateRepository.SaveAsync(aggregate);

			return Ok();
		}
	}
}
