﻿using EventStoreExample.Core.Events;
using EventStoreExample.Core.Framework;
using System;

namespace EventStoreExample.Core
{
	public class TechItem : Aggregate
	{
		public string Name { get; private set; }

		public string Description { get; private set; }

		public string ActiveColumn { get; private set; }

		public string AssignedTo { get; private set; }

		public bool IsCompleted { get; private set; }
		protected override void When(object @event)
		{
			switch (@event)
			{
				case TechItemCreated t:
					OnCreated(t);
					break;
				case TechItemAssigned t:
					OnAssigned(t);
					break;
				case TechItemMoved t:
					OnMoved(t);
					break;
				case TechItemCompleted t:
					OnCompleted(t);
					break;
				default:
					break;
			}
		}

		public void Create(Guid id, string title, string createdBy)
		{
			if (Version >= 0)
			{
				throw new TechItemCreatedException();
			}

			Apply(new TechItemCreated
			{
				Id = id,
				TItle = title,
				CreatedBy = createdBy
			});
		}

		public void Assign(string assignedTo, string assignedBy)
		{
			if (Version == -1)
			{
				throw new TechItemNotFoundException();
			}

			if (IsCompleted)
			{
				throw new TechItemCompletedException();
			}

			Apply(new TechItemAssigned
			{
				Id = Id,
				AssignedBy = assignedBy,
				AssignedTo = assignedTo,
			});
		}

		public void Move(string activeColumn, string movedBy)
		{
			if (Version == -1)
			{
				throw new TechItemNotFoundException();
			}

			if (IsCompleted)
			{
				throw new TechItemCompletedException();
			}

			Apply(new TechItemMoved
			{
				Id = Id,
				MovedBy = movedBy,
				ActiveColumn = activeColumn,
			});
		}

		public void Complete(string completedBy)
		{
			if (Version == -1)
			{
				throw new TechItemNotFoundException();
			}

			if (IsCompleted)
			{
				throw new TechItemCompletedException();
			}

			Apply(new TechItemCompleted
			{
				Id = Id,
				CompletedBy = completedBy
			});
		}

		#region Event Handlers

		private void OnCreated(TechItemCreated @event)
		{
			Id = @event.Id;
			Name = @event.TItle;
			ActiveColumn = "Ready for Development";
		}

		private void OnAssigned(TechItemAssigned @event)
		{
			AssignedTo = @event.AssignedTo;
		}

		private void OnMoved(TechItemMoved @event)
		{
			ActiveColumn = @event.ActiveColumn;
		}

		private void OnCompleted(TechItemCompleted @event)
		{
			IsCompleted = true;
		}

		#endregion
	}
}
